# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from .cors import (
    CORSResource,
    AsyncCORSResource,
    CORSResourceWithRawResponse,
    AsyncCORSResourceWithRawResponse,
    CORSResourceWithStreamingResponse,
    AsyncCORSResourceWithStreamingResponse,
)
from .locks import (
    LocksResource,
    AsyncLocksResource,
    LocksResourceWithRawResponse,
    AsyncLocksResourceWithRawResponse,
    LocksResourceWithStreamingResponse,
    AsyncLocksResourceWithStreamingResponse,
)
from .sippy import (
    SippyResource,
    AsyncSippyResource,
    SippyResourceWithRawResponse,
    AsyncSippyResourceWithRawResponse,
    SippyResourceWithStreamingResponse,
    AsyncSippyResourceWithStreamingResponse,
)
from .metrics import (
    MetricsResource,
    AsyncMetricsResource,
    MetricsResourceWithRawResponse,
    AsyncMetricsResourceWithRawResponse,
    MetricsResourceWithStreamingResponse,
    AsyncMetricsResourceWithStreamingResponse,
)
from .objects import (
    ObjectsResource,
    AsyncObjectsResource,
    ObjectsResourceWithRawResponse,
    AsyncObjectsResourceWithRawResponse,
    ObjectsResourceWithStreamingResponse,
    AsyncObjectsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import is_given, path_template, maybe_transform, strip_not_given, async_maybe_transform
from .lifecycle import (
    LifecycleResource,
    AsyncLifecycleResource,
    LifecycleResourceWithRawResponse,
    AsyncLifecycleResourceWithRawResponse,
    LifecycleResourceWithStreamingResponse,
    AsyncLifecycleResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ....types.r2 import bucket_list_params, bucket_create_params
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from .domains.domains import (
    DomainsResource,
    AsyncDomainsResource,
    DomainsResourceWithRawResponse,
    AsyncDomainsResourceWithRawResponse,
    DomainsResourceWithStreamingResponse,
    AsyncDomainsResourceWithStreamingResponse,
)
from ....types.r2.bucket import Bucket
from .event_notifications import (
    EventNotificationsResource,
    AsyncEventNotificationsResource,
    EventNotificationsResourceWithRawResponse,
    AsyncEventNotificationsResourceWithRawResponse,
    EventNotificationsResourceWithStreamingResponse,
    AsyncEventNotificationsResourceWithStreamingResponse,
)
from ....types.r2.bucket_list_response import BucketListResponse

__all__ = ["BucketsResource", "AsyncBucketsResource"]


class BucketsResource(SyncAPIResource):
    @cached_property
    def lifecycle(self) -> LifecycleResource:
        return LifecycleResource(self._client)

    @cached_property
    def cors(self) -> CORSResource:
        return CORSResource(self._client)

    @cached_property
    def domains(self) -> DomainsResource:
        return DomainsResource(self._client)

    @cached_property
    def event_notifications(self) -> EventNotificationsResource:
        return EventNotificationsResource(self._client)

    @cached_property
    def locks(self) -> LocksResource:
        return LocksResource(self._client)

    @cached_property
    def metrics(self) -> MetricsResource:
        return MetricsResource(self._client)

    @cached_property
    def sippy(self) -> SippyResource:
        return SippyResource(self._client)

    @cached_property
    def objects(self) -> ObjectsResource:
        return ObjectsResource(self._client)

    @cached_property
    def with_raw_response(self) -> BucketsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return BucketsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BucketsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return BucketsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        location_hint: Literal["apac", "eeur", "enam", "weur", "wnam", "oc"] | Omit = omit,
        storage_class: Literal["Standard", "InfrequentAccess"] | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Bucket:
        """
        Creates a new R2 bucket.

        Args:
          account_id: Account ID.

          name: Name of the bucket.

          location_hint: Location of the bucket.

          storage_class: Storage class for newly uploaded objects, unless specified otherwise.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._post(
            path_template("/accounts/{account_id}/r2/buckets", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "location_hint": location_hint,
                    "storage_class": storage_class,
                },
                bucket_create_params.BucketCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Bucket]._unwrapper,
            ),
            cast_to=cast(Type[Bucket], ResultWrapper[Bucket]),
        )

    def list(
        self,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        name_contains: str | Omit = omit,
        order: Literal["name"] | Omit = omit,
        per_page: float | Omit = omit,
        start_after: str | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BucketListResponse:
        """
        Lists all R2 buckets on your account.

        Args:
          account_id: Account ID.

          cursor: Pagination cursor received during the last List Buckets call. R2 buckets are
              paginated using cursors instead of page numbers.

          direction: Direction to order buckets.

          name_contains: Bucket names to filter by. Only buckets with this phrase in their name will be
              returned.

          order: Field to order buckets by.

          per_page: Maximum number of buckets to return in a single call.

          start_after: Bucket name to start searching after. Buckets are ordered lexicographically.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._get(
            path_template("/accounts/{account_id}/r2/buckets", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "direction": direction,
                        "name_contains": name_contains,
                        "order": order,
                        "per_page": per_page,
                        "start_after": start_after,
                    },
                    bucket_list_params.BucketListParams,
                ),
                post_parser=ResultWrapper[BucketListResponse]._unwrapper,
            ),
            cast_to=cast(Type[BucketListResponse], ResultWrapper[BucketListResponse]),
        )

    def delete(
        self,
        bucket_name: str,
        *,
        account_id: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._delete(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def edit(
        self,
        bucket_name: str,
        *,
        account_id: str,
        storage_class: Literal["Standard", "InfrequentAccess"],
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Bucket:
        """
        Updates properties of an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          storage_class: Storage class for newly uploaded objects, unless specified otherwise.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given(
                {
                    "cf-r2-storage-class": str(storage_class),
                    "cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given,
                }
            ),
            **(extra_headers or {}),
        }
        return self._patch(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Bucket]._unwrapper,
            ),
            cast_to=cast(Type[Bucket], ResultWrapper[Bucket]),
        )

    def get(
        self,
        bucket_name: str,
        *,
        account_id: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Bucket:
        """
        Gets properties of an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return self._get(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Bucket]._unwrapper,
            ),
            cast_to=cast(Type[Bucket], ResultWrapper[Bucket]),
        )


class AsyncBucketsResource(AsyncAPIResource):
    @cached_property
    def lifecycle(self) -> AsyncLifecycleResource:
        return AsyncLifecycleResource(self._client)

    @cached_property
    def cors(self) -> AsyncCORSResource:
        return AsyncCORSResource(self._client)

    @cached_property
    def domains(self) -> AsyncDomainsResource:
        return AsyncDomainsResource(self._client)

    @cached_property
    def event_notifications(self) -> AsyncEventNotificationsResource:
        return AsyncEventNotificationsResource(self._client)

    @cached_property
    def locks(self) -> AsyncLocksResource:
        return AsyncLocksResource(self._client)

    @cached_property
    def metrics(self) -> AsyncMetricsResource:
        return AsyncMetricsResource(self._client)

    @cached_property
    def sippy(self) -> AsyncSippyResource:
        return AsyncSippyResource(self._client)

    @cached_property
    def objects(self) -> AsyncObjectsResource:
        return AsyncObjectsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBucketsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBucketsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBucketsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncBucketsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        location_hint: Literal["apac", "eeur", "enam", "weur", "wnam", "oc"] | Omit = omit,
        storage_class: Literal["Standard", "InfrequentAccess"] | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Bucket:
        """
        Creates a new R2 bucket.

        Args:
          account_id: Account ID.

          name: Name of the bucket.

          location_hint: Location of the bucket.

          storage_class: Storage class for newly uploaded objects, unless specified otherwise.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return await self._post(
            path_template("/accounts/{account_id}/r2/buckets", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "location_hint": location_hint,
                    "storage_class": storage_class,
                },
                bucket_create_params.BucketCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Bucket]._unwrapper,
            ),
            cast_to=cast(Type[Bucket], ResultWrapper[Bucket]),
        )

    async def list(
        self,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        direction: Literal["asc", "desc"] | Omit = omit,
        name_contains: str | Omit = omit,
        order: Literal["name"] | Omit = omit,
        per_page: float | Omit = omit,
        start_after: str | Omit = omit,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BucketListResponse:
        """
        Lists all R2 buckets on your account.

        Args:
          account_id: Account ID.

          cursor: Pagination cursor received during the last List Buckets call. R2 buckets are
              paginated using cursors instead of page numbers.

          direction: Direction to order buckets.

          name_contains: Bucket names to filter by. Only buckets with this phrase in their name will be
              returned.

          order: Field to order buckets by.

          per_page: Maximum number of buckets to return in a single call.

          start_after: Bucket name to start searching after. Buckets are ordered lexicographically.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return await self._get(
            path_template("/accounts/{account_id}/r2/buckets", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cursor": cursor,
                        "direction": direction,
                        "name_contains": name_contains,
                        "order": order,
                        "per_page": per_page,
                        "start_after": start_after,
                    },
                    bucket_list_params.BucketListParams,
                ),
                post_parser=ResultWrapper[BucketListResponse]._unwrapper,
            ),
            cast_to=cast(Type[BucketListResponse], ResultWrapper[BucketListResponse]),
        )

    async def delete(
        self,
        bucket_name: str,
        *,
        account_id: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return await self._delete(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def edit(
        self,
        bucket_name: str,
        *,
        account_id: str,
        storage_class: Literal["Standard", "InfrequentAccess"],
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Bucket:
        """
        Updates properties of an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          storage_class: Storage class for newly uploaded objects, unless specified otherwise.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given(
                {
                    "cf-r2-storage-class": str(storage_class),
                    "cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given,
                }
            ),
            **(extra_headers or {}),
        }
        return await self._patch(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Bucket]._unwrapper,
            ),
            cast_to=cast(Type[Bucket], ResultWrapper[Bucket]),
        )

    async def get(
        self,
        bucket_name: str,
        *,
        account_id: str,
        jurisdiction: Literal["default", "eu", "fedramp"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Bucket:
        """
        Gets properties of an existing R2 bucket.

        Args:
          account_id: Account ID.

          bucket_name: Name of the bucket.

          jurisdiction: Jurisdiction where objects in this bucket are guaranteed to be stored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {
            **strip_not_given({"cf-r2-jurisdiction": str(jurisdiction) if is_given(jurisdiction) else not_given}),
            **(extra_headers or {}),
        }
        return await self._get(
            path_template(
                "/accounts/{account_id}/r2/buckets/{bucket_name}", account_id=account_id, bucket_name=bucket_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Bucket]._unwrapper,
            ),
            cast_to=cast(Type[Bucket], ResultWrapper[Bucket]),
        )


class BucketsResourceWithRawResponse:
    def __init__(self, buckets: BucketsResource) -> None:
        self._buckets = buckets

        self.create = to_raw_response_wrapper(
            buckets.create,
        )
        self.list = to_raw_response_wrapper(
            buckets.list,
        )
        self.delete = to_raw_response_wrapper(
            buckets.delete,
        )
        self.edit = to_raw_response_wrapper(
            buckets.edit,
        )
        self.get = to_raw_response_wrapper(
            buckets.get,
        )

    @cached_property
    def lifecycle(self) -> LifecycleResourceWithRawResponse:
        return LifecycleResourceWithRawResponse(self._buckets.lifecycle)

    @cached_property
    def cors(self) -> CORSResourceWithRawResponse:
        return CORSResourceWithRawResponse(self._buckets.cors)

    @cached_property
    def domains(self) -> DomainsResourceWithRawResponse:
        return DomainsResourceWithRawResponse(self._buckets.domains)

    @cached_property
    def event_notifications(self) -> EventNotificationsResourceWithRawResponse:
        return EventNotificationsResourceWithRawResponse(self._buckets.event_notifications)

    @cached_property
    def locks(self) -> LocksResourceWithRawResponse:
        return LocksResourceWithRawResponse(self._buckets.locks)

    @cached_property
    def metrics(self) -> MetricsResourceWithRawResponse:
        return MetricsResourceWithRawResponse(self._buckets.metrics)

    @cached_property
    def sippy(self) -> SippyResourceWithRawResponse:
        return SippyResourceWithRawResponse(self._buckets.sippy)

    @cached_property
    def objects(self) -> ObjectsResourceWithRawResponse:
        return ObjectsResourceWithRawResponse(self._buckets.objects)


class AsyncBucketsResourceWithRawResponse:
    def __init__(self, buckets: AsyncBucketsResource) -> None:
        self._buckets = buckets

        self.create = async_to_raw_response_wrapper(
            buckets.create,
        )
        self.list = async_to_raw_response_wrapper(
            buckets.list,
        )
        self.delete = async_to_raw_response_wrapper(
            buckets.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            buckets.edit,
        )
        self.get = async_to_raw_response_wrapper(
            buckets.get,
        )

    @cached_property
    def lifecycle(self) -> AsyncLifecycleResourceWithRawResponse:
        return AsyncLifecycleResourceWithRawResponse(self._buckets.lifecycle)

    @cached_property
    def cors(self) -> AsyncCORSResourceWithRawResponse:
        return AsyncCORSResourceWithRawResponse(self._buckets.cors)

    @cached_property
    def domains(self) -> AsyncDomainsResourceWithRawResponse:
        return AsyncDomainsResourceWithRawResponse(self._buckets.domains)

    @cached_property
    def event_notifications(self) -> AsyncEventNotificationsResourceWithRawResponse:
        return AsyncEventNotificationsResourceWithRawResponse(self._buckets.event_notifications)

    @cached_property
    def locks(self) -> AsyncLocksResourceWithRawResponse:
        return AsyncLocksResourceWithRawResponse(self._buckets.locks)

    @cached_property
    def metrics(self) -> AsyncMetricsResourceWithRawResponse:
        return AsyncMetricsResourceWithRawResponse(self._buckets.metrics)

    @cached_property
    def sippy(self) -> AsyncSippyResourceWithRawResponse:
        return AsyncSippyResourceWithRawResponse(self._buckets.sippy)

    @cached_property
    def objects(self) -> AsyncObjectsResourceWithRawResponse:
        return AsyncObjectsResourceWithRawResponse(self._buckets.objects)


class BucketsResourceWithStreamingResponse:
    def __init__(self, buckets: BucketsResource) -> None:
        self._buckets = buckets

        self.create = to_streamed_response_wrapper(
            buckets.create,
        )
        self.list = to_streamed_response_wrapper(
            buckets.list,
        )
        self.delete = to_streamed_response_wrapper(
            buckets.delete,
        )
        self.edit = to_streamed_response_wrapper(
            buckets.edit,
        )
        self.get = to_streamed_response_wrapper(
            buckets.get,
        )

    @cached_property
    def lifecycle(self) -> LifecycleResourceWithStreamingResponse:
        return LifecycleResourceWithStreamingResponse(self._buckets.lifecycle)

    @cached_property
    def cors(self) -> CORSResourceWithStreamingResponse:
        return CORSResourceWithStreamingResponse(self._buckets.cors)

    @cached_property
    def domains(self) -> DomainsResourceWithStreamingResponse:
        return DomainsResourceWithStreamingResponse(self._buckets.domains)

    @cached_property
    def event_notifications(self) -> EventNotificationsResourceWithStreamingResponse:
        return EventNotificationsResourceWithStreamingResponse(self._buckets.event_notifications)

    @cached_property
    def locks(self) -> LocksResourceWithStreamingResponse:
        return LocksResourceWithStreamingResponse(self._buckets.locks)

    @cached_property
    def metrics(self) -> MetricsResourceWithStreamingResponse:
        return MetricsResourceWithStreamingResponse(self._buckets.metrics)

    @cached_property
    def sippy(self) -> SippyResourceWithStreamingResponse:
        return SippyResourceWithStreamingResponse(self._buckets.sippy)

    @cached_property
    def objects(self) -> ObjectsResourceWithStreamingResponse:
        return ObjectsResourceWithStreamingResponse(self._buckets.objects)


class AsyncBucketsResourceWithStreamingResponse:
    def __init__(self, buckets: AsyncBucketsResource) -> None:
        self._buckets = buckets

        self.create = async_to_streamed_response_wrapper(
            buckets.create,
        )
        self.list = async_to_streamed_response_wrapper(
            buckets.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            buckets.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            buckets.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            buckets.get,
        )

    @cached_property
    def lifecycle(self) -> AsyncLifecycleResourceWithStreamingResponse:
        return AsyncLifecycleResourceWithStreamingResponse(self._buckets.lifecycle)

    @cached_property
    def cors(self) -> AsyncCORSResourceWithStreamingResponse:
        return AsyncCORSResourceWithStreamingResponse(self._buckets.cors)

    @cached_property
    def domains(self) -> AsyncDomainsResourceWithStreamingResponse:
        return AsyncDomainsResourceWithStreamingResponse(self._buckets.domains)

    @cached_property
    def event_notifications(self) -> AsyncEventNotificationsResourceWithStreamingResponse:
        return AsyncEventNotificationsResourceWithStreamingResponse(self._buckets.event_notifications)

    @cached_property
    def locks(self) -> AsyncLocksResourceWithStreamingResponse:
        return AsyncLocksResourceWithStreamingResponse(self._buckets.locks)

    @cached_property
    def metrics(self) -> AsyncMetricsResourceWithStreamingResponse:
        return AsyncMetricsResourceWithStreamingResponse(self._buckets.metrics)

    @cached_property
    def sippy(self) -> AsyncSippyResourceWithStreamingResponse:
        return AsyncSippyResourceWithStreamingResponse(self._buckets.sippy)

    @cached_property
    def objects(self) -> AsyncObjectsResourceWithStreamingResponse:
        return AsyncObjectsResourceWithStreamingResponse(self._buckets.objects)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

from ...._types import SequenceNotStr
from .dns_resolver_settings_v4_param import DNSResolverSettingsV4Param
from .dns_resolver_settings_v6_param import DNSResolverSettingsV6Param

__all__ = [
    "RuleSettingParam",
    "AuditSSH",
    "BISOAdminControls",
    "BlockPage",
    "CheckSession",
    "DNSResolvers",
    "Egress",
    "ForensicCopy",
    "L4override",
    "NotificationSettings",
    "PayloadLog",
    "Quarantine",
    "Redirect",
    "ResolveDNSInternally",
    "UntrustedCERT",
]


class AuditSSH(TypedDict, total=False):
    """Define the settings for the Audit SSH action.

    Settable only for `l4` rules with `audit_ssh` action.
    """

    command_logging: bool
    """Enable SSH command logging."""


class BISOAdminControls(TypedDict, total=False):
    """Configure browser isolation behavior.

    Settable only for `http` rules with the action set to `isolate`.
    """

    copy: Literal["enabled", "disabled", "remote_only"]
    """Configure copy behavior.

    If set to remote_only, users cannot copy isolated content from the remote
    browser to the local clipboard. If this field is absent, copying remains
    enabled. Applies only when version == "v2".
    """

    dcp: bool
    """Set to false to enable copy-pasting. Only applies when `version == "v1"`."""

    dd: bool
    """Set to false to enable downloading. Only applies when `version == "v1"`."""

    dk: bool
    """Set to false to enable keyboard usage. Only applies when `version == "v1"`."""

    download: Literal["enabled", "disabled", "remote_only"]
    """Configure download behavior.

    When set to remote_only, users can view downloads but cannot save them. If this
    field is absent, downloading remains enabled. Applies only when version == "v2".
    """

    dp: bool
    """Set to false to enable printing. Only applies when `version == "v1"`."""

    du: bool
    """Set to false to enable uploading. Only applies when `version == "v1"`."""

    keyboard: Literal["enabled", "disabled"]
    """Configure keyboard usage behavior.

    If this field is absent, keyboard usage remains enabled. Applies only when
    version == "v2".
    """

    paste: Literal["enabled", "disabled", "remote_only"]
    """Configure paste behavior.

    If set to remote_only, users cannot paste content from the local clipboard into
    isolated pages. If this field is absent, pasting remains enabled. Applies only
    when version == "v2".
    """

    printing: Literal["enabled", "disabled"]
    """Configure print behavior.

    Default, Printing is enabled. Applies only when version == "v2".
    """

    upload: Literal["enabled", "disabled"]
    """Configure upload behavior.

    If this field is absent, uploading remains enabled. Applies only when version ==
    "v2".
    """

    version: Literal["v1", "v2"]
    """Indicate which version of the browser isolation controls should apply."""

    wm_id: str
    """Specify the watermark ID (UUID) to apply to the isolated browser session.

    When present, enables watermark rendering in the isolated browser.
    """


class BlockPage(TypedDict, total=False):
    """Configure custom block page settings.

    If missing or null, use the account settings. Settable only for `http` rules with the action set to `block`.
    """

    target_uri: Required[str]
    """Specify the URI to which the user is redirected."""

    include_context: bool
    """Specify whether to pass the context information as query parameters."""


class CheckSession(TypedDict, total=False):
    """Configure session check behavior.

    Settable only for `l4` and `http` rules with the action set to `allow`.
    """

    duration: str
    """Sets the required session freshness threshold.

    The API returns a normalized version of this value.
    """

    enforce: bool
    """Enable session enforcement."""


class DNSResolvers(TypedDict, total=False):
    """Configure custom resolvers to route queries that match the resolver policy.

    Unused with 'resolve_dns_through_cloudflare' or 'resolve_dns_internally' settings. DNS queries get routed to the address closest to their origin. Only valid when a rule's action set to 'resolve'. Settable only for `dns_resolver` rules.
    """

    ipv4: Iterable[DNSResolverSettingsV4Param]

    ipv6: Iterable[DNSResolverSettingsV6Param]


class Egress(TypedDict, total=False):
    """Configure how Gateway Proxy traffic egresses.

    You can enable this setting for rules with Egress actions and filters, or omit it to indicate local egress via WARP IPs. Settable only for `egress` rules.
    """

    ipv4: str
    """Specify the IPv4 address to use for egress."""

    ipv4_fallback: str
    """Specify the fallback IPv4 address to use for egress when the primary IPv4 fails.

    Set '0.0.0.0' to indicate local egress via WARP IPs.
    """

    ipv6: str
    """Specify the IPv6 range to use for egress."""


class ForensicCopy(TypedDict, total=False):
    """
    Configure whether a copy of the HTTP request will be sent to storage when the rule matches.
    """

    enabled: bool
    """Enable sending the copy to storage."""


class L4override(TypedDict, total=False):
    """Send matching traffic to the supplied destination IP address and port.

    Settable only for `l4` rules with the action set to `l4_override`.
    """

    ip: str
    """Defines the IPv4 or IPv6 address."""

    port: int
    """Defines a port number to use for TCP/UDP overrides."""


class NotificationSettings(TypedDict, total=False):
    """Configure a notification to display on the user's device when this rule matched.

    Settable for all types of rules with the action set to `block`.
    """

    enabled: bool
    """Enable notification."""

    include_context: bool
    """Indicates whether to pass the context information as query parameters."""

    msg: str
    """Customize the message shown in the notification."""

    support_url: str
    """Defines an optional URL to direct users to additional information.

    If unset, the notification opens a block page.
    """


class PayloadLog(TypedDict, total=False):
    """Configure DLP payload logging. Settable only for `http` rules."""

    enabled: bool
    """Enable DLP payload logging for this rule."""


class Quarantine(TypedDict, total=False):
    """Configure settings that apply to quarantine rules.

    Settable only for `http` rules.
    """

    file_types: List[
        Literal["exe", "pdf", "doc", "docm", "docx", "rtf", "ppt", "pptx", "xls", "xlsm", "xlsx", "zip", "rar"]
    ]
    """Specify the types of files to sandbox."""


class Redirect(TypedDict, total=False):
    """Apply settings to redirect rules.

    Settable only for `http` rules with the action set to `redirect`.
    """

    target_uri: Required[str]
    """Specify the URI to which the user is redirected."""

    include_context: bool
    """Specify whether to pass the context information as query parameters."""

    preserve_path_and_query: bool
    """
    Specify whether to append the path and query parameters from the original
    request to target_uri.
    """


class ResolveDNSInternally(TypedDict, total=False):
    """
    Configure to forward the query to the internal DNS service, passing the specified 'view_id' as input. Not used when 'dns_resolvers' is specified or 'resolve_dns_through_cloudflare' is set. Only valid when a rule's action set to 'resolve'. Settable only for `dns_resolver` rules.
    """

    fallback: Literal["none", "public_dns"]
    """
    Specify the fallback behavior to apply when the internal DNS response code
    differs from 'NOERROR' or when the response data contains only CNAME records for
    'A' or 'AAAA' queries.
    """

    view_id: str
    """Specify the internal DNS view identifier to pass to the internal DNS service."""


class UntrustedCERT(TypedDict, total=False):
    """
    Configure behavior when an upstream certificate is invalid or an SSL error occurs. Settable only for `http` rules with the action set to `allow`.
    """

    action: Literal["pass_through", "block", "error"]
    """Defines the action performed when an untrusted certificate seen.

    The default action an error with HTTP code 526.
    """


class RuleSettingParam(TypedDict, total=False):
    """Defines settings for this rule.

    Settings apply only to specific rule types and must use compatible selectors. If Terraform detects drift, confirm the setting supports your rule type and check whether the API modifies the value. Use API-returned values in your configuration to prevent drift.
    """

    add_headers: Optional[Dict[str, SequenceNotStr[str]]]
    """Add custom headers to allowed requests as key-value pairs.

    Use header names as keys that map to arrays of header values. Settable only for
    `http` rules with the action set to `allow`.
    """

    allow_child_bypass: Optional[bool]
    """Set to enable MSP children to bypass this rule.

    Only parent MSP accounts can set this. this rule. Settable for all types of
    rules.
    """

    audit_ssh: Optional[AuditSSH]
    """Define the settings for the Audit SSH action.

    Settable only for `l4` rules with `audit_ssh` action.
    """

    biso_admin_controls: BISOAdminControls
    """Configure browser isolation behavior.

    Settable only for `http` rules with the action set to `isolate`.
    """

    block_page: Optional[BlockPage]
    """Configure custom block page settings.

    If missing or null, use the account settings. Settable only for `http` rules
    with the action set to `block`.
    """

    block_page_enabled: bool
    """Enable the custom block page.

    Settable only for `dns` rules with action `block`.
    """

    block_reason: Optional[str]
    """Explain why the rule blocks the request.

    The custom block page shows this text (if enabled). Settable only for `dns`,
    `l4`, and `http` rules when the action set to `block`.
    """

    bypass_parent_rule: Optional[bool]
    """Set to enable MSP accounts to bypass their parent's rules.

    Only MSP child accounts can set this. Settable for all types of rules.
    """

    check_session: Optional[CheckSession]
    """Configure session check behavior.

    Settable only for `l4` and `http` rules with the action set to `allow`.
    """

    dns_resolvers: Optional[DNSResolvers]
    """Configure custom resolvers to route queries that match the resolver policy.

    Unused with 'resolve_dns_through_cloudflare' or 'resolve_dns_internally'
    settings. DNS queries get routed to the address closest to their origin. Only
    valid when a rule's action set to 'resolve'. Settable only for `dns_resolver`
    rules.
    """

    egress: Optional[Egress]
    """Configure how Gateway Proxy traffic egresses.

    You can enable this setting for rules with Egress actions and filters, or omit
    it to indicate local egress via WARP IPs. Settable only for `egress` rules.
    """

    forensic_copy: Optional[ForensicCopy]
    """
    Configure whether a copy of the HTTP request will be sent to storage when the
    rule matches.
    """

    ignore_cname_category_matches: bool
    """Ignore category matches at CNAME domains in a response.

    When off, evaluate categories in this rule against all CNAME domain categories
    in the response. Settable only for `dns` and `dns_resolver` rules.
    """

    insecure_disable_dnssec_validation: bool
    """Specify whether to disable DNSSEC validation (for Allow actions) [INSECURE].

    Settable only for `dns` rules.
    """

    ip_categories: bool
    """Enable IPs in DNS resolver category blocks.

    The system blocks only domain name categories unless you enable this setting.
    Settable only for `dns` and `dns_resolver` rules.
    """

    ip_indicator_feeds: bool
    """Indicates whether to include IPs in DNS resolver indicator feed blocks.

    Default, indicator feeds block only domain names. Settable only for `dns` and
    `dns_resolver` rules.
    """

    l4override: Optional[L4override]
    """Send matching traffic to the supplied destination IP address and port.

    Settable only for `l4` rules with the action set to `l4_override`.
    """

    notification_settings: Optional[NotificationSettings]
    """Configure a notification to display on the user's device when this rule matched.

    Settable for all types of rules with the action set to `block`.
    """

    override_host: str
    """Defines a hostname for override, for the matching DNS queries.

    Settable only for `dns` rules with the action set to `override`.
    """

    override_ips: Optional[SequenceNotStr[str]]
    """Defines a an IP or set of IPs for overriding matched DNS queries.

    Settable only for `dns` rules with the action set to `override`.
    """

    payload_log: Optional[PayloadLog]
    """Configure DLP payload logging. Settable only for `http` rules."""

    quarantine: Optional[Quarantine]
    """Configure settings that apply to quarantine rules.

    Settable only for `http` rules.
    """

    redirect: Optional[Redirect]
    """Apply settings to redirect rules.

    Settable only for `http` rules with the action set to `redirect`.
    """

    resolve_dns_internally: Optional[ResolveDNSInternally]
    """
    Configure to forward the query to the internal DNS service, passing the
    specified 'view_id' as input. Not used when 'dns_resolvers' is specified or
    'resolve_dns_through_cloudflare' is set. Only valid when a rule's action set to
    'resolve'. Settable only for `dns_resolver` rules.
    """

    resolve_dns_through_cloudflare: Optional[bool]
    """
    Enable to send queries that match the policy to Cloudflare's default 1.1.1.1 DNS
    resolver. Cannot set when 'dns_resolvers' specified or 'resolve_dns_internally'
    is set. Only valid when a rule's action set to 'resolve'. Settable only for
    `dns_resolver` rules.
    """

    untrusted_cert: Optional[UntrustedCERT]
    """
    Configure behavior when an upstream certificate is invalid or an SSL error
    occurs. Settable only for `http` rules with the action set to `allow`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .directory.directory import (
    DirectoryResource,
    AsyncDirectoryResource,
    DirectoryResourceWithRawResponse,
    AsyncDirectoryResourceWithRawResponse,
    DirectoryResourceWithStreamingResponse,
    AsyncDirectoryResourceWithStreamingResponse,
)

__all__ = ["ConnectivityResource", "AsyncConnectivityResource"]


class ConnectivityResource(SyncAPIResource):
    @cached_property
    def directory(self) -> DirectoryResource:
        return DirectoryResource(self._client)

    @cached_property
    def with_raw_response(self) -> ConnectivityResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ConnectivityResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConnectivityResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ConnectivityResourceWithStreamingResponse(self)


class AsyncConnectivityResource(AsyncAPIResource):
    @cached_property
    def directory(self) -> AsyncDirectoryResource:
        return AsyncDirectoryResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncConnectivityResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConnectivityResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConnectivityResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncConnectivityResourceWithStreamingResponse(self)


class ConnectivityResourceWithRawResponse:
    def __init__(self, connectivity: ConnectivityResource) -> None:
        self._connectivity = connectivity

    @cached_property
    def directory(self) -> DirectoryResourceWithRawResponse:
        return DirectoryResourceWithRawResponse(self._connectivity.directory)


class AsyncConnectivityResourceWithRawResponse:
    def __init__(self, connectivity: AsyncConnectivityResource) -> None:
        self._connectivity = connectivity

    @cached_property
    def directory(self) -> AsyncDirectoryResourceWithRawResponse:
        return AsyncDirectoryResourceWithRawResponse(self._connectivity.directory)


class ConnectivityResourceWithStreamingResponse:
    def __init__(self, connectivity: ConnectivityResource) -> None:
        self._connectivity = connectivity

    @cached_property
    def directory(self) -> DirectoryResourceWithStreamingResponse:
        return DirectoryResourceWithStreamingResponse(self._connectivity.directory)


class AsyncConnectivityResourceWithStreamingResponse:
    def __init__(self, connectivity: AsyncConnectivityResource) -> None:
        self._connectivity = connectivity

    @cached_property
    def directory(self) -> AsyncDirectoryResourceWithStreamingResponse:
        return AsyncDirectoryResourceWithStreamingResponse(self._connectivity.directory)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "ProxyEndpointCreateParams",
    "ZeroTrustGatewayProxyEndpointIPCreate",
    "ZeroTrustGatewayProxyEndpointIdentityCreate",
]


class ZeroTrustGatewayProxyEndpointIPCreate(TypedDict, total=False):
    account_id: Required[str]

    name: Required[str]
    """Specify the name of the proxy endpoint."""

    kind: Literal["ip"]
    """The proxy endpoint kind"""


class ZeroTrustGatewayProxyEndpointIdentityCreate(TypedDict, total=False):
    account_id: Required[str]

    kind: Required[Literal["identity"]]
    """The proxy endpoint kind"""

    name: Required[str]
    """Specify the name of the proxy endpoint."""


ProxyEndpointCreateParams: TypeAlias = Union[
    ZeroTrustGatewayProxyEndpointIPCreate, ZeroTrustGatewayProxyEndpointIdentityCreate
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ....._types import Body, Query, Headers, NotGiven, not_given
from ....._utils import path_template
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_custom_raw_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ....._base_client import make_request_options

__all__ = ["DownloadsResource", "AsyncDownloadsResource"]


class DownloadsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DownloadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DownloadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DownloadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DownloadsResourceWithStreamingResponse(self)

    def get(
        self,
        filename: str,
        *,
        account_id: str,
        command_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """Downloads artifacts for an executed command.

        Bulk downloads are not supported

        Args:
          command_id: Unique identifier for a command

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not command_id:
            raise ValueError(f"Expected a non-empty value for `command_id` but received {command_id!r}")
        if not filename:
            raise ValueError(f"Expected a non-empty value for `filename` but received {filename!r}")
        extra_headers = {"Accept": "application/zip", **(extra_headers or {})}
        return self._get(
            path_template(
                "/accounts/{account_id}/dex/commands/{command_id}/downloads/{filename}",
                account_id=account_id,
                command_id=command_id,
                filename=filename,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )


class AsyncDownloadsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDownloadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDownloadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDownloadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDownloadsResourceWithStreamingResponse(self)

    async def get(
        self,
        filename: str,
        *,
        account_id: str,
        command_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """Downloads artifacts for an executed command.

        Bulk downloads are not supported

        Args:
          command_id: Unique identifier for a command

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not command_id:
            raise ValueError(f"Expected a non-empty value for `command_id` but received {command_id!r}")
        if not filename:
            raise ValueError(f"Expected a non-empty value for `filename` but received {filename!r}")
        extra_headers = {"Accept": "application/zip", **(extra_headers or {})}
        return await self._get(
            path_template(
                "/accounts/{account_id}/dex/commands/{command_id}/downloads/{filename}",
                account_id=account_id,
                command_id=command_id,
                filename=filename,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )


class DownloadsResourceWithRawResponse:
    def __init__(self, downloads: DownloadsResource) -> None:
        self._downloads = downloads

        self.get = to_custom_raw_response_wrapper(
            downloads.get,
            BinaryAPIResponse,
        )


class AsyncDownloadsResourceWithRawResponse:
    def __init__(self, downloads: AsyncDownloadsResource) -> None:
        self._downloads = downloads

        self.get = async_to_custom_raw_response_wrapper(
            downloads.get,
            AsyncBinaryAPIResponse,
        )


class DownloadsResourceWithStreamingResponse:
    def __init__(self, downloads: DownloadsResource) -> None:
        self._downloads = downloads

        self.get = to_custom_streamed_response_wrapper(
            downloads.get,
            StreamedBinaryAPIResponse,
        )


class AsyncDownloadsResourceWithStreamingResponse:
    def __init__(self, downloads: AsyncDownloadsResource) -> None:
        self._downloads = downloads

        self.get = async_to_custom_streamed_response_wrapper(
            downloads.get,
            AsyncStreamedBinaryAPIResponse,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from .config import (
    ConfigResource,
    AsyncConfigResource,
    ConfigResourceWithRawResponse,
    AsyncConfigResourceWithRawResponse,
    ConfigResourceWithStreamingResponse,
    AsyncConfigResourceWithStreamingResponse,
)
from ....._types import Body, Query, Headers, NotGiven, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.ai_gateway.billing import topup_create_params, topup_status_params
from .....types.ai_gateway.billing.topup_create_response import TopupCreateResponse
from .....types.ai_gateway.billing.topup_status_response import TopupStatusResponse

__all__ = ["TopupResource", "AsyncTopupResource"]


class TopupResource(SyncAPIResource):
    @cached_property
    def config(self) -> ConfigResource:
        return ConfigResource(self._client)

    @cached_property
    def with_raw_response(self) -> TopupResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TopupResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TopupResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TopupResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        amount: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopupCreateResponse:
        """
        Create a credit top-up via Stripe PaymentIntent for the given account.

        Args:
          amount: Top-up amount in cents (min 1000).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/ai-gateway/billing/topup", account_id=account_id),
            body=maybe_transform({"amount": amount}, topup_create_params.TopupCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TopupCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopupCreateResponse], ResultWrapper[TopupCreateResponse]),
        )

    def status(
        self,
        *,
        account_id: str,
        payment_intent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopupStatusResponse:
        """
        Get the payment processing status of a top-up by its invoice ID.

        Args:
          payment_intent_id: Stripe invoice ID to check status for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/ai-gateway/billing/topup/status", account_id=account_id),
            body=maybe_transform({"payment_intent_id": payment_intent_id}, topup_status_params.TopupStatusParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TopupStatusResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopupStatusResponse], ResultWrapper[TopupStatusResponse]),
        )


class AsyncTopupResource(AsyncAPIResource):
    @cached_property
    def config(self) -> AsyncConfigResource:
        return AsyncConfigResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTopupResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTopupResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTopupResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTopupResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        amount: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopupCreateResponse:
        """
        Create a credit top-up via Stripe PaymentIntent for the given account.

        Args:
          amount: Top-up amount in cents (min 1000).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/ai-gateway/billing/topup", account_id=account_id),
            body=await async_maybe_transform({"amount": amount}, topup_create_params.TopupCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TopupCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopupCreateResponse], ResultWrapper[TopupCreateResponse]),
        )

    async def status(
        self,
        *,
        account_id: str,
        payment_intent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TopupStatusResponse:
        """
        Get the payment processing status of a top-up by its invoice ID.

        Args:
          payment_intent_id: Stripe invoice ID to check status for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/ai-gateway/billing/topup/status", account_id=account_id),
            body=await async_maybe_transform(
                {"payment_intent_id": payment_intent_id}, topup_status_params.TopupStatusParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[TopupStatusResponse]._unwrapper,
            ),
            cast_to=cast(Type[TopupStatusResponse], ResultWrapper[TopupStatusResponse]),
        )


class TopupResourceWithRawResponse:
    def __init__(self, topup: TopupResource) -> None:
        self._topup = topup

        self.create = to_raw_response_wrapper(
            topup.create,
        )
        self.status = to_raw_response_wrapper(
            topup.status,
        )

    @cached_property
    def config(self) -> ConfigResourceWithRawResponse:
        return ConfigResourceWithRawResponse(self._topup.config)


class AsyncTopupResourceWithRawResponse:
    def __init__(self, topup: AsyncTopupResource) -> None:
        self._topup = topup

        self.create = async_to_raw_response_wrapper(
            topup.create,
        )
        self.status = async_to_raw_response_wrapper(
            topup.status,
        )

    @cached_property
    def config(self) -> AsyncConfigResourceWithRawResponse:
        return AsyncConfigResourceWithRawResponse(self._topup.config)


class TopupResourceWithStreamingResponse:
    def __init__(self, topup: TopupResource) -> None:
        self._topup = topup

        self.create = to_streamed_response_wrapper(
            topup.create,
        )
        self.status = to_streamed_response_wrapper(
            topup.status,
        )

    @cached_property
    def config(self) -> ConfigResourceWithStreamingResponse:
        return ConfigResourceWithStreamingResponse(self._topup.config)


class AsyncTopupResourceWithStreamingResponse:
    def __init__(self, topup: AsyncTopupResource) -> None:
        self._topup = topup

        self.create = async_to_streamed_response_wrapper(
            topup.create,
        )
        self.status = async_to_streamed_response_wrapper(
            topup.status,
        )

    @cached_property
    def config(self) -> AsyncConfigResourceWithStreamingResponse:
        return AsyncConfigResourceWithStreamingResponse(self._topup.config)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["TLSSettingsParam"]


class TLSSettingsParam(TypedDict, total=False):
    """Specify whether to inspect encrypted HTTP traffic."""

    enabled: bool
    """Specify whether to inspect encrypted HTTP traffic."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .insights import (
    InsightsResource,
    AsyncInsightsResource,
    InsightsResourceWithRawResponse,
    AsyncInsightsResourceWithRawResponse,
    InsightsResourceWithStreamingResponse,
    AsyncInsightsResourceWithStreamingResponse,
)
from .security_center import (
    SecurityCenterResource,
    AsyncSecurityCenterResource,
    SecurityCenterResourceWithRawResponse,
    AsyncSecurityCenterResourceWithRawResponse,
    SecurityCenterResourceWithStreamingResponse,
    AsyncSecurityCenterResourceWithStreamingResponse,
)

__all__ = [
    "InsightsResource",
    "AsyncInsightsResource",
    "InsightsResourceWithRawResponse",
    "AsyncInsightsResourceWithRawResponse",
    "InsightsResourceWithStreamingResponse",
    "AsyncInsightsResourceWithStreamingResponse",
    "SecurityCenterResource",
    "AsyncSecurityCenterResource",
    "SecurityCenterResourceWithRawResponse",
    "AsyncSecurityCenterResourceWithRawResponse",
    "SecurityCenterResourceWithStreamingResponse",
    "AsyncSecurityCenterResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["IdentityProviderSCIMConfigParam"]


class IdentityProviderSCIMConfigParam(TypedDict, total=False):
    """
    The configuration settings for enabling a System for Cross-Domain Identity Management (SCIM) with the identity provider.
    """

    enabled: bool
    """A flag to enable or disable SCIM for the identity provider."""

    identity_update_behavior: Literal["automatic", "reauth", "no_action"]
    """Indicates how a SCIM event updates a user identity used for policy evaluation.

    Use "automatic" to automatically update a user's identity and augment it with
    fields from the SCIM user resource. Use "reauth" to force re-authentication on
    group membership updates, user identity update will only occur after successful
    re-authentication. With "reauth" identities will not contain fields from the
    SCIM user resource. With "no_action" identities will not be changed by SCIM
    updates in any way and users will not be prompted to reauthenticate.
    """

    seat_deprovision: bool
    """
    A flag to remove a user's seat in Zero Trust when they have been deprovisioned
    in the Identity Provider. This cannot be enabled unless user_deprovision is also
    enabled.
    """

    user_deprovision: bool
    """
    A flag to enable revoking a user's session in Access and Gateway when they have
    been deprovisioned in the Identity Provider.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .keys import Keys as Keys
from .audio import Audio as Audio
from .video import Video as Video
from .caption import Caption as Caption
from .watermark import Watermark as Watermark
from .live_input import LiveInput as LiveInput
from .allowed_origins import AllowedOrigins as AllowedOrigins
from .key_get_response import KeyGetResponse as KeyGetResponse
from .key_create_params import KeyCreateParams as KeyCreateParams
from .clip_create_params import ClipCreateParams as ClipCreateParams
from .copy_create_params import CopyCreateParams as CopyCreateParams
from .embed_get_response import EmbedGetResponse as EmbedGetResponse
from .stream_edit_params import StreamEditParams as StreamEditParams
from .stream_list_params import StreamListParams as StreamListParams
from .key_delete_response import KeyDeleteResponse as KeyDeleteResponse
from .token_create_params import TokenCreateParams as TokenCreateParams
from .stream_create_params import StreamCreateParams as StreamCreateParams
from .webhook_get_response import WebhookGetResponse as WebhookGetResponse
from .download_get_response import DownloadGetResponse as DownloadGetResponse
from .token_create_response import TokenCreateResponse as TokenCreateResponse
from .webhook_update_params import WebhookUpdateParams as WebhookUpdateParams
from .live_input_list_params import LiveInputListParams as LiveInputListParams
from .audio_track_copy_params import AudioTrackCopyParams as AudioTrackCopyParams
from .audio_track_edit_params import AudioTrackEditParams as AudioTrackEditParams
from .watermark_create_params import WatermarkCreateParams as WatermarkCreateParams
from .webhook_delete_response import WebhookDeleteResponse as WebhookDeleteResponse
from .webhook_update_response import WebhookUpdateResponse as WebhookUpdateResponse
from .audio_track_get_response import AudioTrackGetResponse as AudioTrackGetResponse
from .download_create_response import DownloadCreateResponse as DownloadCreateResponse
from .download_delete_response import DownloadDeleteResponse as DownloadDeleteResponse
from .live_input_create_params import LiveInputCreateParams as LiveInputCreateParams
from .live_input_list_response import LiveInputListResponse as LiveInputListResponse
from .live_input_update_params import LiveInputUpdateParams as LiveInputUpdateParams
from .watermark_delete_response import WatermarkDeleteResponse as WatermarkDeleteResponse
from .video_storage_usage_params import VideoStorageUsageParams as VideoStorageUsageParams
from .audio_track_delete_response import AudioTrackDeleteResponse as AudioTrackDeleteResponse
from .direct_upload_create_params import DirectUploadCreateParams as DirectUploadCreateParams
from .video_storage_usage_response import VideoStorageUsageResponse as VideoStorageUsageResponse
from .direct_upload_create_response import DirectUploadCreateResponse as DirectUploadCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ......_types import Body, Query, Headers, NotGiven, SequenceNotStr, not_given
from ......_utils import path_template, maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_wrappers import ResultWrapper
from ......pagination import SyncSinglePage, AsyncSinglePage
from ......_base_client import AsyncPaginator, make_request_options
from ......types.workers_for_platforms.dispatch.namespaces.scripts.tag_list_response import TagListResponse
from ......types.workers_for_platforms.dispatch.namespaces.scripts.tag_update_response import TagUpdateResponse

__all__ = ["TagsResource", "AsyncTagsResource"]


class TagsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TagsResourceWithStreamingResponse(self)

    def update(
        self,
        script_name: str,
        *,
        account_id: str,
        dispatch_namespace: str,
        body: Optional[SequenceNotStr[str]],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[TagUpdateResponse]:
        """
        Put script tags for a script uploaded to a Workers for Platforms namespace.

        Args:
          account_id: Identifier.

          dispatch_namespace: Name of the Workers for Platforms dispatch namespace.

          script_name: Name of the script, used in URLs and route configuration.

          body: Tags associated with the Worker.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dispatch_namespace:
            raise ValueError(f"Expected a non-empty value for `dispatch_namespace` but received {dispatch_namespace!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workers/dispatch/namespaces/{dispatch_namespace}/scripts/{script_name}/tags",
                account_id=account_id,
                dispatch_namespace=dispatch_namespace,
                script_name=script_name,
            ),
            page=SyncSinglePage[TagUpdateResponse],
            body=maybe_transform(body, Optional[SequenceNotStr[str]]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=str,
            method="put",
        )

    def list(
        self,
        script_name: str,
        *,
        account_id: str,
        dispatch_namespace: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[TagListResponse]:
        """
        Fetch tags from a script uploaded to a Workers for Platforms namespace.

        Args:
          account_id: Identifier.

          dispatch_namespace: Name of the Workers for Platforms dispatch namespace.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dispatch_namespace:
            raise ValueError(f"Expected a non-empty value for `dispatch_namespace` but received {dispatch_namespace!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workers/dispatch/namespaces/{dispatch_namespace}/scripts/{script_name}/tags",
                account_id=account_id,
                dispatch_namespace=dispatch_namespace,
                script_name=script_name,
            ),
            page=SyncSinglePage[TagListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=str,
        )

    def delete(
        self,
        tag: str,
        *,
        account_id: str,
        dispatch_namespace: str,
        script_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete script tag for a script uploaded to a Workers for Platforms namespace.

        Args:
          account_id: Identifier.

          dispatch_namespace: Name of the Workers for Platforms dispatch namespace.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dispatch_namespace:
            raise ValueError(f"Expected a non-empty value for `dispatch_namespace` but received {dispatch_namespace!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        if not tag:
            raise ValueError(f"Expected a non-empty value for `tag` but received {tag!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/workers/dispatch/namespaces/{dispatch_namespace}/scripts/{script_name}/tags/{tag}",
                account_id=account_id,
                dispatch_namespace=dispatch_namespace,
                script_name=script_name,
                tag=tag,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class AsyncTagsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTagsResourceWithStreamingResponse(self)

    def update(
        self,
        script_name: str,
        *,
        account_id: str,
        dispatch_namespace: str,
        body: Optional[SequenceNotStr[str]],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TagUpdateResponse, AsyncSinglePage[TagUpdateResponse]]:
        """
        Put script tags for a script uploaded to a Workers for Platforms namespace.

        Args:
          account_id: Identifier.

          dispatch_namespace: Name of the Workers for Platforms dispatch namespace.

          script_name: Name of the script, used in URLs and route configuration.

          body: Tags associated with the Worker.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dispatch_namespace:
            raise ValueError(f"Expected a non-empty value for `dispatch_namespace` but received {dispatch_namespace!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workers/dispatch/namespaces/{dispatch_namespace}/scripts/{script_name}/tags",
                account_id=account_id,
                dispatch_namespace=dispatch_namespace,
                script_name=script_name,
            ),
            page=AsyncSinglePage[TagUpdateResponse],
            body=maybe_transform(body, Optional[SequenceNotStr[str]]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=str,
            method="put",
        )

    def list(
        self,
        script_name: str,
        *,
        account_id: str,
        dispatch_namespace: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TagListResponse, AsyncSinglePage[TagListResponse]]:
        """
        Fetch tags from a script uploaded to a Workers for Platforms namespace.

        Args:
          account_id: Identifier.

          dispatch_namespace: Name of the Workers for Platforms dispatch namespace.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dispatch_namespace:
            raise ValueError(f"Expected a non-empty value for `dispatch_namespace` but received {dispatch_namespace!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workers/dispatch/namespaces/{dispatch_namespace}/scripts/{script_name}/tags",
                account_id=account_id,
                dispatch_namespace=dispatch_namespace,
                script_name=script_name,
            ),
            page=AsyncSinglePage[TagListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=str,
        )

    async def delete(
        self,
        tag: str,
        *,
        account_id: str,
        dispatch_namespace: str,
        script_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete script tag for a script uploaded to a Workers for Platforms namespace.

        Args:
          account_id: Identifier.

          dispatch_namespace: Name of the Workers for Platforms dispatch namespace.

          script_name: Name of the script, used in URLs and route configuration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not dispatch_namespace:
            raise ValueError(f"Expected a non-empty value for `dispatch_namespace` but received {dispatch_namespace!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        if not tag:
            raise ValueError(f"Expected a non-empty value for `tag` but received {tag!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/workers/dispatch/namespaces/{dispatch_namespace}/scripts/{script_name}/tags/{tag}",
                account_id=account_id,
                dispatch_namespace=dispatch_namespace,
                script_name=script_name,
                tag=tag,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class TagsResourceWithRawResponse:
    def __init__(self, tags: TagsResource) -> None:
        self._tags = tags

        self.update = to_raw_response_wrapper(
            tags.update,
        )
        self.list = to_raw_response_wrapper(
            tags.list,
        )
        self.delete = to_raw_response_wrapper(
            tags.delete,
        )


class AsyncTagsResourceWithRawResponse:
    def __init__(self, tags: AsyncTagsResource) -> None:
        self._tags = tags

        self.update = async_to_raw_response_wrapper(
            tags.update,
        )
        self.list = async_to_raw_response_wrapper(
            tags.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tags.delete,
        )


class TagsResourceWithStreamingResponse:
    def __init__(self, tags: TagsResource) -> None:
        self._tags = tags

        self.update = to_streamed_response_wrapper(
            tags.update,
        )
        self.list = to_streamed_response_wrapper(
            tags.list,
        )
        self.delete = to_streamed_response_wrapper(
            tags.delete,
        )


class AsyncTagsResourceWithStreamingResponse:
    def __init__(self, tags: AsyncTagsResource) -> None:
        self._tags = tags

        self.update = async_to_streamed_response_wrapper(
            tags.update,
        )
        self.list = async_to_streamed_response_wrapper(
            tags.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tags.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ACLUpdateParams"]


class ACLUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    ip_range: Required[str]
    """Allowed IPv4/IPv6 address range of primary or secondary nameservers.

    This will be applied for the entire account. The IP range is used to allow
    additional NOTIFY IPs for secondary zones and IPs Cloudflare allows AXFR/IXFR
    requests from for primary zones. CIDRs are limited to a maximum of /24 for IPv4
    and /64 for IPv6 respectively.
    """

    name: Required[str]
    """The name of the acl."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

from ....._types import SequenceNotStr

__all__ = ["PredefinedUpdateParams", "Entry"]


class PredefinedUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    ai_context_enabled: bool

    allowed_match_count: Optional[int]

    confidence_threshold: Optional[str]

    enabled_entries: Optional[SequenceNotStr[str]]

    entries: Iterable[Entry]

    ocr_enabled: bool


class Entry(TypedDict, total=False):
    id: Required[str]

    enabled: Required[bool]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["WARPDeleteResponse"]


class WARPDeleteResponse(BaseModel):
    id: Optional[str] = None
    """The UUID of the subnet."""

    comment: Optional[str] = None
    """An optional description of the subnet."""

    created_at: Optional[datetime] = None
    """Timestamp of when the resource was created."""

    deleted_at: Optional[datetime] = None
    """Timestamp of when the resource was deleted.

    If `null`, the resource has not been deleted.
    """

    is_default_network: Optional[bool] = None
    """If `true`, this is the default subnet for the account.

    There can only be one default subnet per account.
    """

    name: Optional[str] = None
    """A user-friendly name for the subnet."""

    network: Optional[str] = None
    """The private IPv4 or IPv6 range defining the subnet, in CIDR notation."""

    subnet_type: Optional[Literal["cloudflare_source", "warp"]] = None
    """The type of subnet."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["StatusEditParams", "From"]


class StatusEditParams(TypedDict, total=False):
    account_id: Required[str]

    workflow_name: Required[str]

    status: Required[Literal["resume", "pause", "terminate", "restart"]]
    """Apply action to instance."""

    from_: Annotated[From, PropertyInfo(alias="from")]
    """Step to restart from. Only applicable when status is "restart"."""


class From(TypedDict, total=False):
    """Step to restart from. Only applicable when status is "restart"."""

    name: Required[str]

    count: int

    type: Literal["do", "sleep", "waitForEvent"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .summary import (
    SummaryResource,
    AsyncSummaryResource,
    SummaryResourceWithRawResponse,
    AsyncSummaryResourceWithRawResponse,
    SummaryResourceWithStreamingResponse,
    AsyncSummaryResourceWithStreamingResponse,
)
from .inference import (
    InferenceResource,
    AsyncInferenceResource,
    InferenceResourceWithRawResponse,
    AsyncInferenceResourceWithRawResponse,
    InferenceResourceWithStreamingResponse,
    AsyncInferenceResourceWithStreamingResponse,
)
from .timeseries_groups import (
    TimeseriesGroupsResource,
    AsyncTimeseriesGroupsResource,
    TimeseriesGroupsResourceWithRawResponse,
    AsyncTimeseriesGroupsResourceWithRawResponse,
    TimeseriesGroupsResourceWithStreamingResponse,
    AsyncTimeseriesGroupsResourceWithStreamingResponse,
)

__all__ = [
    "SummaryResource",
    "AsyncSummaryResource",
    "SummaryResourceWithRawResponse",
    "AsyncSummaryResourceWithRawResponse",
    "SummaryResourceWithStreamingResponse",
    "AsyncSummaryResourceWithStreamingResponse",
    "TimeseriesGroupsResource",
    "AsyncTimeseriesGroupsResource",
    "TimeseriesGroupsResourceWithRawResponse",
    "AsyncTimeseriesGroupsResourceWithRawResponse",
    "TimeseriesGroupsResourceWithStreamingResponse",
    "AsyncTimeseriesGroupsResourceWithStreamingResponse",
    "InferenceResource",
    "AsyncInferenceResource",
    "InferenceResourceWithRawResponse",
    "AsyncInferenceResourceWithRawResponse",
    "InferenceResourceWithStreamingResponse",
    "AsyncInferenceResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = [
    "AIGatewayUpdateParams",
    "DLP",
    "DLPUnionMember0",
    "DLPUnionMember1",
    "DLPUnionMember1Policy",
    "Guardrails",
    "GuardrailsPrompt",
    "GuardrailsResponse",
    "Otel",
    "Stripe",
    "StripeUsageEvent",
]


class AIGatewayUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    cache_invalidate_on_update: Required[bool]

    cache_ttl: Required[Optional[int]]

    collect_logs: Required[bool]

    rate_limiting_interval: Required[Optional[int]]

    rate_limiting_limit: Required[Optional[int]]

    authentication: bool

    dlp: DLP

    guardrails: Optional[Guardrails]

    log_management: Optional[int]

    log_management_strategy: Optional[Literal["STOP_INSERTING", "DELETE_OLDEST"]]

    logpush: bool

    logpush_public_key: Optional[str]

    otel: Optional[Iterable[Otel]]

    rate_limiting_technique: Optional[Literal["fixed", "sliding"]]

    retry_backoff: Optional[Literal["constant", "linear", "exponential"]]
    """Backoff strategy for retry delays"""

    retry_delay: Optional[int]
    """Delay between retry attempts in milliseconds (0-5000)"""

    retry_max_attempts: Optional[int]
    """Maximum number of retry attempts for failed requests (1-5)"""

    store_id: Optional[str]

    stripe: Optional[Stripe]

    workers_ai_billing_mode: Literal["postpaid"]
    """Controls how Workers AI inference calls routed through this gateway are billed.

    Only 'postpaid' is currently supported.
    """

    zdr: bool


class DLPUnionMember0(TypedDict, total=False):
    action: Required[Literal["BLOCK", "FLAG"]]

    enabled: Required[bool]

    profiles: Required[SequenceNotStr[str]]


class DLPUnionMember1Policy(TypedDict, total=False):
    id: Required[str]

    action: Required[Literal["FLAG", "BLOCK"]]

    check: Required[List[Literal["REQUEST", "RESPONSE"]]]

    enabled: Required[bool]

    profiles: Required[SequenceNotStr[str]]


class DLPUnionMember1(TypedDict, total=False):
    enabled: Required[bool]

    policies: Required[Iterable[DLPUnionMember1Policy]]


DLP: TypeAlias = Union[DLPUnionMember0, DLPUnionMember1]


class GuardrailsPrompt(TypedDict, total=False):
    p1: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="P1")]

    s1: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S1")]

    s10: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S10")]

    s11: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S11")]

    s12: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S12")]

    s13: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S13")]

    s2: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S2")]

    s3: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S3")]

    s4: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S4")]

    s5: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S5")]

    s6: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S6")]

    s7: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S7")]

    s8: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S8")]

    s9: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S9")]


class GuardrailsResponse(TypedDict, total=False):
    p1: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="P1")]

    s1: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S1")]

    s10: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S10")]

    s11: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S11")]

    s12: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S12")]

    s13: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S13")]

    s2: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S2")]

    s3: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S3")]

    s4: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S4")]

    s5: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S5")]

    s6: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S6")]

    s7: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S7")]

    s8: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S8")]

    s9: Annotated[Literal["FLAG", "BLOCK"], PropertyInfo(alias="S9")]


class Guardrails(TypedDict, total=False):
    prompt: Required[GuardrailsPrompt]

    response: Required[GuardrailsResponse]


class Otel(TypedDict, total=False):
    authorization: Required[str]

    headers: Required[Dict[str, str]]

    url: Required[str]

    content_type: Literal["json", "protobuf"]


class StripeUsageEvent(TypedDict, total=False):
    payload: Required[str]


class Stripe(TypedDict, total=False):
    authorization: Required[str]

    usage_events: Required[Iterable[StripeUsageEvent]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union
from typing_extensions import Literal, TypeAlias

__all__ = ["SettingValue"]

SettingValue: TypeAlias = Union[Literal["1.0", "1.1", "1.2", "1.3", "on", "off"], List[str]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .sippy import Sippy as Sippy
from .provider import Provider as Provider
from .cors_get_response import CORSGetResponse as CORSGetResponse
from .lock_get_response import LockGetResponse as LockGetResponse
from .cors_update_params import CORSUpdateParams as CORSUpdateParams
from .lock_update_params import LockUpdateParams as LockUpdateParams
from .object_list_params import ObjectListParams as ObjectListParams
from .sippy_update_params import SippyUpdateParams as SippyUpdateParams
from .metric_list_response import MetricListResponse as MetricListResponse
from .object_list_response import ObjectListResponse as ObjectListResponse
from .object_upload_params import ObjectUploadParams as ObjectUploadParams
from .sippy_delete_response import SippyDeleteResponse as SippyDeleteResponse
from .lifecycle_get_response import LifecycleGetResponse as LifecycleGetResponse
from .object_delete_response import ObjectDeleteResponse as ObjectDeleteResponse
from .object_upload_response import ObjectUploadResponse as ObjectUploadResponse
from .lifecycle_update_params import LifecycleUpdateParams as LifecycleUpdateParams
from .event_notification_get_response import EventNotificationGetResponse as EventNotificationGetResponse
from .event_notification_list_response import EventNotificationListResponse as EventNotificationListResponse
from .event_notification_update_params import EventNotificationUpdateParams as EventNotificationUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ItemListParams"]


class ItemListParams(TypedDict, total=False):
    account_id: Required[str]
    """The Account ID for this resource."""

    cursor: str
    """The pagination cursor.

    An opaque string token indicating the position from which to continue when
    requesting the next/previous set of records. Cursor values are provided under
    `result_info.cursors` in the response. You should make no assumptions about a
    cursor's content or length.
    """

    per_page: int
    """Amount of results to include in each paginated response.

    A non-negative 32 bit integer.
    """

    search: str
    """A search query to filter returned items.

    Its meaning depends on the list type: IP addresses must start with the provided
    string, hostnames and bulk redirects must contain the string, and ASNs must
    match the string exactly.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["CfInterconnectBulkUpdateParams"]


class CfInterconnectBulkUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    body: Required[object]

    x_magic_new_hc_target: Annotated[bool, PropertyInfo(alias="x-magic-new-hc-target")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .domains import (
    DomainsResource,
    AsyncDomainsResource,
    DomainsResourceWithRawResponse,
    AsyncDomainsResourceWithRawResponse,
    DomainsResourceWithStreamingResponse,
    AsyncDomainsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .block_senders import (
    BlockSendersResource,
    AsyncBlockSendersResource,
    BlockSendersResourceWithRawResponse,
    AsyncBlockSendersResourceWithRawResponse,
    BlockSendersResourceWithStreamingResponse,
    AsyncBlockSendersResourceWithStreamingResponse,
)
from .allow_policies import (
    AllowPoliciesResource,
    AsyncAllowPoliciesResource,
    AllowPoliciesResourceWithRawResponse,
    AsyncAllowPoliciesResourceWithRawResponse,
    AllowPoliciesResourceWithStreamingResponse,
    AsyncAllowPoliciesResourceWithStreamingResponse,
)
from .trusted_domains import (
    TrustedDomainsResource,
    AsyncTrustedDomainsResource,
    TrustedDomainsResourceWithRawResponse,
    AsyncTrustedDomainsResourceWithRawResponse,
    TrustedDomainsResourceWithStreamingResponse,
    AsyncTrustedDomainsResourceWithStreamingResponse,
)
from .impersonation_registry import (
    ImpersonationRegistryResource,
    AsyncImpersonationRegistryResource,
    ImpersonationRegistryResourceWithRawResponse,
    AsyncImpersonationRegistryResourceWithRawResponse,
    ImpersonationRegistryResourceWithStreamingResponse,
    AsyncImpersonationRegistryResourceWithStreamingResponse,
)

__all__ = ["SettingsResource", "AsyncSettingsResource"]


class SettingsResource(SyncAPIResource):
    @cached_property
    def allow_policies(self) -> AllowPoliciesResource:
        return AllowPoliciesResource(self._client)

    @cached_property
    def block_senders(self) -> BlockSendersResource:
        return BlockSendersResource(self._client)

    @cached_property
    def domains(self) -> DomainsResource:
        return DomainsResource(self._client)

    @cached_property
    def impersonation_registry(self) -> ImpersonationRegistryResource:
        return ImpersonationRegistryResource(self._client)

    @cached_property
    def trusted_domains(self) -> TrustedDomainsResource:
        return TrustedDomainsResource(self._client)

    @cached_property
    def with_raw_response(self) -> SettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SettingsResourceWithStreamingResponse(self)


class AsyncSettingsResource(AsyncAPIResource):
    @cached_property
    def allow_policies(self) -> AsyncAllowPoliciesResource:
        return AsyncAllowPoliciesResource(self._client)

    @cached_property
    def block_senders(self) -> AsyncBlockSendersResource:
        return AsyncBlockSendersResource(self._client)

    @cached_property
    def domains(self) -> AsyncDomainsResource:
        return AsyncDomainsResource(self._client)

    @cached_property
    def impersonation_registry(self) -> AsyncImpersonationRegistryResource:
        return AsyncImpersonationRegistryResource(self._client)

    @cached_property
    def trusted_domains(self) -> AsyncTrustedDomainsResource:
        return AsyncTrustedDomainsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSettingsResourceWithStreamingResponse(self)


class SettingsResourceWithRawResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

    @cached_property
    def allow_policies(self) -> AllowPoliciesResourceWithRawResponse:
        return AllowPoliciesResourceWithRawResponse(self._settings.allow_policies)

    @cached_property
    def block_senders(self) -> BlockSendersResourceWithRawResponse:
        return BlockSendersResourceWithRawResponse(self._settings.block_senders)

    @cached_property
    def domains(self) -> DomainsResourceWithRawResponse:
        return DomainsResourceWithRawResponse(self._settings.domains)

    @cached_property
    def impersonation_registry(self) -> ImpersonationRegistryResourceWithRawResponse:
        return ImpersonationRegistryResourceWithRawResponse(self._settings.impersonation_registry)

    @cached_property
    def trusted_domains(self) -> TrustedDomainsResourceWithRawResponse:
        return TrustedDomainsResourceWithRawResponse(self._settings.trusted_domains)


class AsyncSettingsResourceWithRawResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

    @cached_property
    def allow_policies(self) -> AsyncAllowPoliciesResourceWithRawResponse:
        return AsyncAllowPoliciesResourceWithRawResponse(self._settings.allow_policies)

    @cached_property
    def block_senders(self) -> AsyncBlockSendersResourceWithRawResponse:
        return AsyncBlockSendersResourceWithRawResponse(self._settings.block_senders)

    @cached_property
    def domains(self) -> AsyncDomainsResourceWithRawResponse:
        return AsyncDomainsResourceWithRawResponse(self._settings.domains)

    @cached_property
    def impersonation_registry(self) -> AsyncImpersonationRegistryResourceWithRawResponse:
        return AsyncImpersonationRegistryResourceWithRawResponse(self._settings.impersonation_registry)

    @cached_property
    def trusted_domains(self) -> AsyncTrustedDomainsResourceWithRawResponse:
        return AsyncTrustedDomainsResourceWithRawResponse(self._settings.trusted_domains)


class SettingsResourceWithStreamingResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

    @cached_property
    def allow_policies(self) -> AllowPoliciesResourceWithStreamingResponse:
        return AllowPoliciesResourceWithStreamingResponse(self._settings.allow_policies)

    @cached_property
    def block_senders(self) -> BlockSendersResourceWithStreamingResponse:
        return BlockSendersResourceWithStreamingResponse(self._settings.block_senders)

    @cached_property
    def domains(self) -> DomainsResourceWithStreamingResponse:
        return DomainsResourceWithStreamingResponse(self._settings.domains)

    @cached_property
    def impersonation_registry(self) -> ImpersonationRegistryResourceWithStreamingResponse:
        return ImpersonationRegistryResourceWithStreamingResponse(self._settings.impersonation_registry)

    @cached_property
    def trusted_domains(self) -> TrustedDomainsResourceWithStreamingResponse:
        return TrustedDomainsResourceWithStreamingResponse(self._settings.trusted_domains)


class AsyncSettingsResourceWithStreamingResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

    @cached_property
    def allow_policies(self) -> AsyncAllowPoliciesResourceWithStreamingResponse:
        return AsyncAllowPoliciesResourceWithStreamingResponse(self._settings.allow_policies)

    @cached_property
    def block_senders(self) -> AsyncBlockSendersResourceWithStreamingResponse:
        return AsyncBlockSendersResourceWithStreamingResponse(self._settings.block_senders)

    @cached_property
    def domains(self) -> AsyncDomainsResourceWithStreamingResponse:
        return AsyncDomainsResourceWithStreamingResponse(self._settings.domains)

    @cached_property
    def impersonation_registry(self) -> AsyncImpersonationRegistryResourceWithStreamingResponse:
        return AsyncImpersonationRegistryResourceWithStreamingResponse(self._settings.impersonation_registry)

    @cached_property
    def trusted_domains(self) -> AsyncTrustedDomainsResourceWithStreamingResponse:
        return AsyncTrustedDomainsResourceWithStreamingResponse(self._settings.trusted_domains)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["SchemaDeleteResponse"]


class SchemaDeleteResponse(BaseModel):
    id: str
    """The ID of the schema that was just deleted"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncSinglePage, AsyncSinglePage
from ....._base_client import AsyncPaginator, make_request_options
from .....types.zero_trust.access.certificates import setting_update_params
from .....types.zero_trust.access.certificates.certificate_settings import CertificateSettings
from .....types.zero_trust.access.certificates.certificate_settings_param import CertificateSettingsParam

__all__ = ["SettingsResource", "AsyncSettingsResource"]


class SettingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SettingsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        settings: Iterable[CertificateSettingsParam],
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[CertificateSettings]:
        """
        Updates an mTLS certificate's hostname settings.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/access/certificates/settings",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncSinglePage[CertificateSettings],
            body=maybe_transform({"settings": settings}, setting_update_params.SettingUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CertificateSettings,
            method="put",
        )

    def get(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[CertificateSettings]:
        """
        List all mTLS hostname settings for this account or zone.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/access/certificates/settings",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncSinglePage[CertificateSettings],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CertificateSettings,
        )


class AsyncSettingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSettingsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        settings: Iterable[CertificateSettingsParam],
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CertificateSettings, AsyncSinglePage[CertificateSettings]]:
        """
        Updates an mTLS certificate's hostname settings.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/access/certificates/settings",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncSinglePage[CertificateSettings],
            body=maybe_transform({"settings": settings}, setting_update_params.SettingUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CertificateSettings,
            method="put",
        )

    def get(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CertificateSettings, AsyncSinglePage[CertificateSettings]]:
        """
        List all mTLS hostname settings for this account or zone.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/access/certificates/settings",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncSinglePage[CertificateSettings],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CertificateSettings,
        )


class SettingsResourceWithRawResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.update = to_raw_response_wrapper(
            settings.update,
        )
        self.get = to_raw_response_wrapper(
            settings.get,
        )


class AsyncSettingsResourceWithRawResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.update = async_to_raw_response_wrapper(
            settings.update,
        )
        self.get = async_to_raw_response_wrapper(
            settings.get,
        )


class SettingsResourceWithStreamingResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.update = to_streamed_response_wrapper(
            settings.update,
        )
        self.get = to_streamed_response_wrapper(
            settings.get,
        )


class AsyncSettingsResourceWithStreamingResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.update = async_to_streamed_response_wrapper(
            settings.update,
        )
        self.get = async_to_streamed_response_wrapper(
            settings.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .cmb.cmb import (
    CmbResource,
    AsyncCmbResource,
    CmbResourceWithRawResponse,
    AsyncCmbResourceWithRawResponse,
    CmbResourceWithStreamingResponse,
    AsyncCmbResourceWithStreamingResponse,
)
from .retention import (
    RetentionResource,
    AsyncRetentionResource,
    RetentionResourceWithRawResponse,
    AsyncRetentionResourceWithRawResponse,
    RetentionResourceWithStreamingResponse,
    AsyncRetentionResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["ControlResource", "AsyncControlResource"]


class ControlResource(SyncAPIResource):
    @cached_property
    def retention(self) -> RetentionResource:
        return RetentionResource(self._client)

    @cached_property
    def cmb(self) -> CmbResource:
        return CmbResource(self._client)

    @cached_property
    def with_raw_response(self) -> ControlResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ControlResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ControlResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ControlResourceWithStreamingResponse(self)


class AsyncControlResource(AsyncAPIResource):
    @cached_property
    def retention(self) -> AsyncRetentionResource:
        return AsyncRetentionResource(self._client)

    @cached_property
    def cmb(self) -> AsyncCmbResource:
        return AsyncCmbResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncControlResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncControlResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncControlResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncControlResourceWithStreamingResponse(self)


class ControlResourceWithRawResponse:
    def __init__(self, control: ControlResource) -> None:
        self._control = control

    @cached_property
    def retention(self) -> RetentionResourceWithRawResponse:
        return RetentionResourceWithRawResponse(self._control.retention)

    @cached_property
    def cmb(self) -> CmbResourceWithRawResponse:
        return CmbResourceWithRawResponse(self._control.cmb)


class AsyncControlResourceWithRawResponse:
    def __init__(self, control: AsyncControlResource) -> None:
        self._control = control

    @cached_property
    def retention(self) -> AsyncRetentionResourceWithRawResponse:
        return AsyncRetentionResourceWithRawResponse(self._control.retention)

    @cached_property
    def cmb(self) -> AsyncCmbResourceWithRawResponse:
        return AsyncCmbResourceWithRawResponse(self._control.cmb)


class ControlResourceWithStreamingResponse:
    def __init__(self, control: ControlResource) -> None:
        self._control = control

    @cached_property
    def retention(self) -> RetentionResourceWithStreamingResponse:
        return RetentionResourceWithStreamingResponse(self._control.retention)

    @cached_property
    def cmb(self) -> CmbResourceWithStreamingResponse:
        return CmbResourceWithStreamingResponse(self._control.cmb)


class AsyncControlResourceWithStreamingResponse:
    def __init__(self, control: AsyncControlResource) -> None:
        self._control = control

    @cached_property
    def retention(self) -> AsyncRetentionResourceWithStreamingResponse:
        return AsyncRetentionResourceWithStreamingResponse(self._control.retention)

    @cached_property
    def cmb(self) -> AsyncCmbResourceWithStreamingResponse:
        return AsyncCmbResourceWithStreamingResponse(self._control.cmb)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["GlobalWARPOverrideCreateParams"]


class GlobalWARPOverrideCreateParams(TypedDict, total=False):
    account_id: Required[str]

    disconnect: Required[bool]
    """Disconnects all devices on the account using Global WARP override."""

    justification: str
    """Reasoning for setting the Global WARP override state.

    This will be surfaced in the audit log.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = ["NetworkPathGetResponse", "Hop", "HopLocation"]


class HopLocation(BaseModel):
    city: Optional[str] = None

    state: Optional[str] = None

    zip: Optional[str] = None


class Hop(BaseModel):
    ttl: int

    asn: Optional[int] = None

    aso: Optional[str] = None

    ip_address: Optional[str] = FieldInfo(alias="ipAddress", default=None)

    location: Optional[HopLocation] = None

    mile: Optional[Literal["client-to-app", "client-to-cf-egress", "client-to-cf-ingress", "client-to-isp"]] = None

    name: Optional[str] = None

    packet_loss_pct: Optional[float] = FieldInfo(alias="packetLossPct", default=None)

    rtt_ms: Optional[int] = FieldInfo(alias="rttMs", default=None)


class NetworkPathGetResponse(BaseModel):
    hops: List[Hop]
    """an array of the hops taken by the device to reach the end destination"""

    result_id: str = FieldInfo(alias="resultId")
    """API Resource UUID tag."""

    device_name: Optional[str] = FieldInfo(alias="deviceName", default=None)
    """name of the device associated with this network path response"""

    test_id: Optional[str] = FieldInfo(alias="testId", default=None)
    """API Resource UUID tag."""

    test_name: Optional[str] = FieldInfo(alias="testName", default=None)
    """name of the tracroute test"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from ....._types import SequenceNotStr
from ....._utils import PropertyInfo

__all__ = ["SummaryHTTPMethodParams"]


class SummaryHTTPMethodParams(TypedDict, total=False):
    asn: SequenceNotStr[str]
    """Filters results by Autonomous System.

    Specify one or more Autonomous System Numbers (ASNs) as a comma-separated list.
    Prefix with `-` to exclude ASNs from results. For example, `-174, 3356` excludes
    results from AS174, but includes results from AS3356.
    """

    continent: SequenceNotStr[str]
    """Filters results by continent.

    Specify a comma-separated list of alpha-2 codes. Prefix with `-` to exclude
    continents from results. For example, `-EU,NA` excludes results from EU, but
    includes results from NA.
    """

    date_end: Annotated[SequenceNotStr[Union[str, datetime]], PropertyInfo(alias="dateEnd", format="iso8601")]
    """End of the date range (inclusive)."""

    date_range: Annotated[SequenceNotStr[str], PropertyInfo(alias="dateRange")]
    """Filters results by date range.

    For example, use `7d` and `7dcontrol` to compare this week with the previous
    week. Use this parameter or set specific start and end dates (`dateStart` and
    `dateEnd` parameters).
    """

    date_start: Annotated[SequenceNotStr[Union[str, datetime]], PropertyInfo(alias="dateStart", format="iso8601")]
    """Start of the date range."""

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    http_version: Annotated[List[Literal["HTTPv1", "HTTPv2", "HTTPv3"]], PropertyInfo(alias="httpVersion")]
    """Filters results by HTTP version."""

    ip_version: Annotated[List[Literal["IPv4", "IPv6"]], PropertyInfo(alias="ipVersion")]
    """Filters results by IP version (Ipv4 vs. IPv6)."""

    limit_per_group: Annotated[int, PropertyInfo(alias="limitPerGroup")]
    """
    Limits the number of objects per group to the top items within the specified
    time range. When item count exceeds the limit, extra items appear grouped under
    an "other" category.
    """

    location: SequenceNotStr[str]
    """Filters results by location.

    Specify a comma-separated list of alpha-2 codes. Prefix with `-` to exclude
    locations from results. For example, `-US,PT` excludes results from the US, but
    includes results from PT.
    """

    mitigation_product: Annotated[
        List[
            Literal[
                "DDOS", "WAF", "BOT_MANAGEMENT", "ACCESS_RULES", "IP_REPUTATION", "API_SHIELD", "DATA_LOSS_PREVENTION"
            ]
        ],
        PropertyInfo(alias="mitigationProduct"),
    ]
    """Filters the results by layer 7 mitigation product."""

    name: SequenceNotStr[str]
    """Array of names used to label the series in the response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ThreatEventBulkCreateResponse", "CreatedEvent", "Error"]


class CreatedEvent(BaseModel):
    event_index: float = FieldInfo(alias="eventIndex")
    """Original index in the input data array"""

    shard_id: str = FieldInfo(alias="shardId")
    """Dataset ID of the shard where the event was created"""

    uuid: str
    """UUID of the created event"""


class Error(BaseModel):
    error: str
    """Error message"""

    event_index: float = FieldInfo(alias="eventIndex")
    """Index of the event that caused the error"""


class ThreatEventBulkCreateResponse(BaseModel):
    """Detailed result of bulk event creation with auto-tag management"""

    created_events_count: float = FieldInfo(alias="createdEventsCount")
    """Number of events created"""

    created_tags_count: float = FieldInfo(alias="createdTagsCount")
    """Number of new tags created in SoT"""

    error_count: float = FieldInfo(alias="errorCount")
    """Number of errors encountered"""

    queued_indicators_count: float = FieldInfo(alias="queuedIndicatorsCount")
    """Number of indicators queued for async processing"""

    create_bulk_events_request_id: Optional[str] = FieldInfo(alias="createBulkEventsRequestId", default=None)
    """Correlation ID for async indicator processing"""

    created_events: Optional[List[CreatedEvent]] = FieldInfo(alias="createdEvents", default=None)
    """Array of created events with UUIDs and shard locations.

    Only present when includeCreatedEvents=true
    """

    errors: Optional[List[Error]] = None
    """Array of error details"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from pydantic import Field as FieldInfo

from ..._utils import PropertyInfo
from ..._models import BaseModel

__all__ = [
    "SinkCreateResponse",
    "Config",
    "ConfigCloudflarePipelinesR2Table",
    "ConfigCloudflarePipelinesR2TableCredentials",
    "ConfigCloudflarePipelinesR2TableFileNaming",
    "ConfigCloudflarePipelinesR2TablePartitioning",
    "ConfigCloudflarePipelinesR2TableRollingPolicy",
    "ConfigCloudflarePipelinesR2DataCatalogTable",
    "ConfigCloudflarePipelinesR2DataCatalogTableRollingPolicy",
    "Format",
    "FormatJson",
    "FormatParquet",
    "Schema",
    "SchemaField",
    "SchemaFieldInt32",
    "SchemaFieldInt64",
    "SchemaFieldFloat32",
    "SchemaFieldFloat64",
    "SchemaFieldBool",
    "SchemaFieldString",
    "SchemaFieldBinary",
    "SchemaFieldTimestamp",
    "SchemaFieldJson",
    "SchemaFieldStruct",
    "SchemaFieldList",
    "SchemaFormat",
    "SchemaFormatJson",
    "SchemaFormatParquet",
]


class ConfigCloudflarePipelinesR2TableCredentials(BaseModel):
    access_key_id: str
    """Cloudflare Account ID for the bucket"""

    secret_access_key: str
    """Cloudflare Account ID for the bucket"""


class ConfigCloudflarePipelinesR2TableFileNaming(BaseModel):
    """Controls filename prefix/suffix and strategy."""

    prefix: Optional[str] = None
    """The prefix to use in file name. i.e prefix-<uuid>.parquet"""

    strategy: Optional[Literal["serial", "uuid", "uuid_v7", "ulid"]] = None
    """Filename generation strategy."""

    suffix: Optional[str] = None
    """This will overwrite the default file suffix. i.e .parquet, use with caution"""


class ConfigCloudflarePipelinesR2TablePartitioning(BaseModel):
    """Data-layout partitioning for sinks."""

    time_pattern: Optional[str] = None
    """The pattern of the date string"""


class ConfigCloudflarePipelinesR2TableRollingPolicy(BaseModel):
    """Rolling policy for file sinks (when & why to close a file and open a new one)."""

    file_size_bytes: Optional[int] = None
    """Files will be rolled after reaching this number of bytes"""

    inactivity_seconds: Optional[int] = None
    """Number of seconds of inactivity to wait before rolling over to a new file"""

    interval_seconds: Optional[int] = None
    """Number of seconds to wait before rolling over to a new file"""


class ConfigCloudflarePipelinesR2Table(BaseModel):
    account_id: str
    """Cloudflare Account ID for the bucket"""

    bucket: str
    """R2 Bucket to write to"""

    credentials: ConfigCloudflarePipelinesR2TableCredentials

    file_naming: Optional[ConfigCloudflarePipelinesR2TableFileNaming] = None
    """Controls filename prefix/suffix and strategy."""

    jurisdiction: Optional[str] = None
    """Jurisdiction this bucket is hosted in"""

    partitioning: Optional[ConfigCloudflarePipelinesR2TablePartitioning] = None
    """Data-layout partitioning for sinks."""

    path: Optional[str] = None
    """Subpath within the bucket to write to"""

    rolling_policy: Optional[ConfigCloudflarePipelinesR2TableRollingPolicy] = None
    """Rolling policy for file sinks (when & why to close a file and open a new one)."""


class ConfigCloudflarePipelinesR2DataCatalogTableRollingPolicy(BaseModel):
    """Rolling policy for file sinks (when & why to close a file and open a new one)."""

    file_size_bytes: Optional[int] = None
    """Files will be rolled after reaching this number of bytes"""

    inactivity_seconds: Optional[int] = None
    """Number of seconds of inactivity to wait before rolling over to a new file"""

    interval_seconds: Optional[int] = None
    """Number of seconds to wait before rolling over to a new file"""


class ConfigCloudflarePipelinesR2DataCatalogTable(BaseModel):
    """R2 Data Catalog Sink"""

    token: str
    """Authentication token"""

    account_id: str
    """Cloudflare Account ID"""

    bucket: str
    """The R2 Bucket that hosts this catalog"""

    table_name: str
    """Table name"""

    namespace: Optional[str] = None
    """Table namespace"""

    rolling_policy: Optional[ConfigCloudflarePipelinesR2DataCatalogTableRollingPolicy] = None
    """Rolling policy for file sinks (when & why to close a file and open a new one)."""


Config: TypeAlias = Union[ConfigCloudflarePipelinesR2Table, ConfigCloudflarePipelinesR2DataCatalogTable]


class FormatJson(BaseModel):
    type: Literal["json"]

    decimal_encoding: Optional[Literal["number", "string", "bytes"]] = None

    timestamp_format: Optional[Literal["rfc3339", "unix_millis"]] = None

    unstructured: Optional[bool] = None


class FormatParquet(BaseModel):
    type: Literal["parquet"]

    compression: Optional[Literal["uncompressed", "snappy", "gzip", "zstd", "lz4"]] = None

    row_group_bytes: Optional[int] = None


Format: TypeAlias = Annotated[Union[FormatJson, FormatParquet], PropertyInfo(discriminator="type")]


class SchemaFieldInt32(BaseModel):
    type: Literal["int32"]

    metadata_key: Optional[str] = None

    name: Optional[str] = None

    required: Optional[bool] = None

    sql_name: Optional[str] = None


class SchemaFieldInt64(BaseModel):
    type: Literal["int64"]

    metadata_key: Optional[str] = None

    name: Optional[str] = None

    required: Optional[bool] = None

    sql_name: Optional[str] = None


class SchemaFieldFloat32(BaseModel):
    type: Literal["float32"]

    metadata_key: Optional[str] = None

    name: Optional[str] = None

    required: Optional[bool] = None

    sql_name: Optional[str] = None


class SchemaFieldFloat64(BaseModel):
    type: Literal["float64"]

    metadata_key: Optional[str] = None

    name: Optional[str] = None

    required: Optional[bool] = None

    sql_name: Optional[str] = None


class SchemaFieldBool(BaseModel):
    type: Literal["bool"]

    metadata_key: Optional[str] = None

    name: Optional[str] = None

    required: Optional[bool] = None

    sql_name: Optional[str] = None


class SchemaFieldString(BaseModel):
    type: Literal["string"]

    metadata_key: Optional[str] = None

    name: Optional[str] = None

    required: Optional[bool] = None

    sql_name: Optional[str] = None


class SchemaFieldBinary(BaseModel):
    type: Literal["binary"]

    metadata_key: Optional[str] = None

    name: Optional[str] = None

    required: Optional[bool] = None

    sql_name: Optional[str] = None


class SchemaFieldTimestamp(BaseModel):
    type: Literal["timestamp"]

    metadata_key: Optional[str] = None

    name: Optional[str] = None

    required: Optional[bool] = None

    sql_name: Optional[str] = None

    unit: Optional[Literal["second", "millisecond", "microsecond", "nanosecond"]] = None


class SchemaFieldJson(BaseModel):
    type: Literal["json"]

    metadata_key: Optional[str] = None

    name: Optional[str] = None

    required: Optional[bool] = None

    sql_name: Optional[str] = None


class SchemaFieldStruct(BaseModel):
    pass


class SchemaFieldList(BaseModel):
    pass


SchemaField: TypeAlias = Annotated[
    Union[
        SchemaFieldInt32,
        SchemaFieldInt64,
        SchemaFieldFloat32,
        SchemaFieldFloat64,
        SchemaFieldBool,
        SchemaFieldString,
        SchemaFieldBinary,
        SchemaFieldTimestamp,
        SchemaFieldJson,
        SchemaFieldStruct,
        SchemaFieldList,
    ],
    PropertyInfo(discriminator="type"),
]


class SchemaFormatJson(BaseModel):
    type: Literal["json"]

    decimal_encoding: Optional[Literal["number", "string", "bytes"]] = None

    timestamp_format: Optional[Literal["rfc3339", "unix_millis"]] = None

    unstructured: Optional[bool] = None


class SchemaFormatParquet(BaseModel):
    type: Literal["parquet"]

    compression: Optional[Literal["uncompressed", "snappy", "gzip", "zstd", "lz4"]] = None

    row_group_bytes: Optional[int] = None


SchemaFormat: TypeAlias = Annotated[Union[SchemaFormatJson, SchemaFormatParquet], PropertyInfo(discriminator="type")]


class Schema(BaseModel):
    fields: Optional[List[SchemaField]] = None

    format: Optional[SchemaFormat] = None

    inferred: Optional[bool] = None


class SinkCreateResponse(BaseModel):
    id: str
    """Indicates a unique identifier for this sink."""

    created_at: datetime

    modified_at: datetime

    name: str
    """Defines the name of the Sink."""

    type: Literal["r2", "r2_data_catalog"]
    """Specifies the type of sink."""

    config: Optional[Config] = None
    """R2 Data Catalog Sink"""

    format: Optional[Format] = None

    schema_: Optional[Schema] = FieldInfo(alias="schema", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DynamicRoutingListParams"]


class DynamicRoutingListParams(TypedDict, total=False):
    account_id: Required[str]

    page: int
    """Page number"""

    per_page: int
    """Number of routes per page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .argo import (
    ArgoResource,
    AsyncArgoResource,
    ArgoResourceWithRawResponse,
    AsyncArgoResourceWithRawResponse,
    ArgoResourceWithStreamingResponse,
    AsyncArgoResourceWithStreamingResponse,
)
from .smart_routing import (
    SmartRoutingResource,
    AsyncSmartRoutingResource,
    SmartRoutingResourceWithRawResponse,
    AsyncSmartRoutingResourceWithRawResponse,
    SmartRoutingResourceWithStreamingResponse,
    AsyncSmartRoutingResourceWithStreamingResponse,
)
from .tiered_caching import (
    TieredCachingResource,
    AsyncTieredCachingResource,
    TieredCachingResourceWithRawResponse,
    AsyncTieredCachingResourceWithRawResponse,
    TieredCachingResourceWithStreamingResponse,
    AsyncTieredCachingResourceWithStreamingResponse,
)

__all__ = [
    "SmartRoutingResource",
    "AsyncSmartRoutingResource",
    "SmartRoutingResourceWithRawResponse",
    "AsyncSmartRoutingResourceWithRawResponse",
    "SmartRoutingResourceWithStreamingResponse",
    "AsyncSmartRoutingResourceWithStreamingResponse",
    "TieredCachingResource",
    "AsyncTieredCachingResource",
    "TieredCachingResourceWithRawResponse",
    "AsyncTieredCachingResourceWithRawResponse",
    "TieredCachingResourceWithStreamingResponse",
    "AsyncTieredCachingResourceWithStreamingResponse",
    "ArgoResource",
    "AsyncArgoResource",
    "ArgoResourceWithRawResponse",
    "AsyncArgoResourceWithRawResponse",
    "ArgoResourceWithStreamingResponse",
    "AsyncArgoResourceWithStreamingResponse",
]
